/*
 * This file contains functions that exist solely to support functionality
 * needed in CUED
 *
 */


#include "cupsd.h"
#include "cups/md5.h"
/*
 * CuedValidateAuth looks at the cued-release-code attribute of the job
 * and checks that it is a valid checksum for the concatenation of a
 * secret (known to this server, and to the "printer accounting front
 * end") and the job-id.
 * If the check succeeds, it returns 1, if it fails, it sets the job
 * hold-until attribute to indefinite and returns 0  .
 * Current implementation creates the checksum by
 * md5 ("secret"."job-id")
 * but this may change in future
 */


#define TESTING_MODE

int                                    /* O - 1 if permitted, 0 otherwise */
CuedValidateAuth(job_t *job)           /* I - job to be inspected */
{
  ipp_attribute_t	*attr;		/* job attribute */
  int failed = 0;
  md5_state_t md5_state;
  md5_byte_t  md5_digest[16];
  char        md5_digest_txt[35];

  /* Set the next #define to teh maximum length of string that the job 
     number integer could require.

     We set the job string to be the job id, padded to at least 
     five digits;  it can't currently be bigger than a maximum
     signed 32bit integer, so 11+1 chars is sufficient (I don't
     particularly want to think what'll happen when jobids become
     negative, but it won't break -here- ) 
  */
         
#define MAXJIDLEN 11

  md5_byte_t  jobid[MAXJIDLEN+1];

  /* end of local variable declarations */

  /* if we've assigned a job to a particular printer, and that printer
     isn't playing the accounting game, then we let it through 
   */
  if (job->printer && !job->printer->cued_accounting)
     return 1;

  /* if we've not assigned a job to a particular printer, find out if
     the queue it's asking for is playing the accounting game
   */
  if (!job->printer) {
     printer_t  *printer;

     if ((printer = FindClass(job->dest)) == NULL)
        printer = FindPrinter(job->dest);	
     if (!printer) {
        /* shouldn't happen, reject attempt */
        return 0;
     }
     if (!printer->cued_accounting) {
        /* we're not doing the cued accounting game, so allow it through */
        return 1;
     }
  }  

  md5_init(&md5_state);

  if (!CuedAccountSecret) {
    LogMessage(L_ERROR, "CuedValidateAuth: no CuedAccountSecret set, can't release any jobs!");
    failed = 1;
  } else {  
    md5_append(&md5_state, (const md5_byte_t *) CuedAccountSecret, 
	       strlen(CuedAccountSecret));
  }


  if (!failed &&
      (((attr = ippFindAttribute(job->attrs, "cued-accounting-user",
				 IPP_TAG_STRING)) != NULL) ||
       ((attr = ippFindAttribute(job->attrs, "cued-accounting-user",
				 IPP_TAG_NAME)) != NULL))) {
    md5_append(&md5_state, 
	       (const md5_byte_t *) attr->values[0].string.text,
	       strlen(attr->values[0].string.text));
    md5_append(&md5_state, (const md5_byte_t *) "|", 1);
    LogMessage(L_DEBUG2, "CuedValidateAuth: cued-accounting-user: %s", 
	       attr->values[0].string.text);
  } else {
    if (!failed) {	
       LogMessage(L_DEBUG, "CuedValidateAuth: failed to find cued-accounting-user");
       failed = 1;
    }
  }

  if (!failed &&
      ((attr = ippFindAttribute(job->attrs, "job-printer-uri",
				 IPP_TAG_URI)) != NULL)) {
    md5_append(&md5_state, 
	       (const md5_byte_t *) attr->values[0].string.text,
	       strlen(attr->values[0].string.text));
    md5_append(&md5_state, (const md5_byte_t *) "|", 1);
    
  } else {
    if (!failed) {	
       LogMessage(L_DEBUG, "CuedValidateAuth: failed to find job-printer-uri");
       failed = 1;
    }   
  }

  if (!failed) {
    memset(jobid, '0', MAXJIDLEN); jobid[MAXJIDLEN] = '\0';
    snprintf((char *)jobid, MAXJIDLEN+1, "%05d", job->id);
    
    md5_append(&md5_state, (const md5_byte_t *) jobid, strlen(jobid));
  }
  
  md5_finish(&md5_state, md5_digest);
    
  snprintf(md5_digest_txt, 35, 
	   "0x"
	   "%02x%02x%02x%02x%02x%02x%02x%02x"
	   "%02x%02x%02x%02x%02x%02x%02x%02x", 
	   md5_digest[0], md5_digest[1], md5_digest[2], md5_digest[3],
	   md5_digest[4], md5_digest[5], md5_digest[6], md5_digest[7],
	   md5_digest[8], md5_digest[9], md5_digest[10], md5_digest[11],
	   md5_digest[12], md5_digest[13], md5_digest[14], md5_digest[15]);
	   

  if (!failed && 
      (((attr = ippFindAttribute(job->attrs, "cued-release-code", 
				 IPP_TAG_STRING)) != NULL) ||
       ((attr = ippFindAttribute(job->attrs, "cued-release-code", 
				 IPP_TAG_NAME)) != NULL))) {
#ifdef TESTING
    if (!strcmp(attr->values[0].string.text, "MontyPython")) {
      LogMessage(L_DEBUG, "CuedValidateAuth:  Magic testing value found"); 
      return 1;
    }
#endif
    
    LogMessage(L_DEBUG2, "CuedValidateAuth: cued-release-code: %s",
	       attr->values[0].string.text);

    if (!strcmp(md5_digest_txt, attr->values[0].string.text)) {
      LogMessage(L_DEBUG, "CuedValidateAuth: valid md5 sum in release code");
      return 1;
    } else {
      failed = 1;
      LogMessage(L_DEBUG, "CuedValidateAuth: md5 sum in release code did not match");
      LogMessage(L_DEBUG2, "CuedValidateAuth: calculated \"%s\", provided \"%s\"", md5_digest_txt, attr->values[0].string.text);
    }
  } else {
    if (!failed) {	
       failed = 1;
       LogMessage(L_DEBUG, "CuedValidateAuth: failed to find cued-release-code");  } 
  }

  /* we failed to find a suitable checksum, so we set indefinite hold,
     and fail */
  while ((attr = ippFindAttribute(job->attrs, "job-hold-until", 
				  IPP_TAG_KEYWORD)))
    ippDeleteAttribute(job->attrs, attr);
  while ((attr = ippFindAttribute(job->attrs, "job-hold-until", IPP_TAG_NAME)))
    ippDeleteAttribute(job->attrs, attr);
  attr = ippAddString(job->attrs, IPP_TAG_JOB, IPP_TAG_KEYWORD,
		      "job-hold-until", NULL,
		      "indefinite");
  
  job->state->values[0].integer = IPP_JOB_HELD;
  SetJobHoldUntil(job->id, attr->values[0].string.text);
  
  LogMessage(L_DEBUG, "attempt to validate job %d for accounting failed, so set to indefinite hold", job->id);
  
  return 0;
}
